require 'spec_helper'

describe ::CrvApiClient::Helpers::Crv do
  let(:tested_class) { Class.new { include ::CrvApiClient::Helpers::Crv }.new }
  let(:participant_code) { '123' }
  context 'get_keeper' do
    it 'should return an ParticipatId class' do
      expect(tested_class).to receive(:endpoint_participant_id).and_return(::CrvApiClient::Api::Animals::ParticipantId)
      result = tested_class.get_keeper("123",nil)
      expect(result).to be_an(::CrvApiClient::Api::Animals::ParticipantId)
    end

    it 'should return netherlands type when country is nil' do
      expect(tested_class).to receive(:endpoint_participant_id).and_return(::CrvApiClient::Api::Animals::ParticipantId)

      result = tested_class.get_keeper(participant_code,nil)
      expect(result.countryCode).to eq("NLD")
      expect(result.participantCode).to be(participant_code)
      expect(result.participantCodeType).to eq("UBN")
    end

    it 'should return netherlands type when country is NL' do
      expect(tested_class).to receive(:endpoint_participant_id).and_return(::CrvApiClient::Api::Animals::ParticipantId)

      result = tested_class.get_keeper(participant_code,"nl")
      expect(result.countryCode).to eq("NLD")
      expect(result.participantCode).to be(participant_code)
      expect(result.participantCodeType).to eq("UBN")
    end

    it 'should return german type when country is DE' do
      expect(tested_class).to receive(:endpoint_participant_id).and_return(::CrvApiClient::Api::Animals::ParticipantId)

      result = tested_class.get_keeper(participant_code,"DE")
      expect(result.countryCode).to eq("DE")
      expect(result.participantCode).to be(participant_code)
      expect(result.participantCodeType).to eq("LDE")
    end

    it 'should return belgian type when country is BE' do
      expect(tested_class).to receive(:endpoint_participant_id).and_return(::CrvApiClient::Api::Animals::ParticipantId)

      result = tested_class.get_keeper(participant_code,"BE")
      expect(result.countryCode).to eq("BE")
      expect(result.participantCode).to be(participant_code)
      expect(result.participantCodeType).to eq("UVN")
    end

    it 'should return czech type when country is CZ' do
      expect(tested_class).to receive(:endpoint_participant_id).and_return(::CrvApiClient::Api::Animals::ParticipantId)

      result = tested_class.get_keeper(participant_code,"CZ")
      expect(result.countryCode).to eq("CZ")
      expect(result.participantCode).to be(participant_code)
      expect(result.participantCodeType).to eq("LCZ")
    end
  end

  context :get_crv_organisation_code do

    it 'should return netherlands crv code when country is nil' do
      result = tested_class.get_crv_organisation_code nil
      expect(result).to eq("crv.nl")
    end

    it 'should return netherlands crv code when country is NL' do
      result = tested_class.get_crv_organisation_code "nl"
      expect(result).to eq("crv.nl")
    end

    it 'should return german crv code when country is DE' do
      result = tested_class.get_crv_organisation_code "DE"
      expect(result).to eq("crv.de")
    end

    it 'should return belgian crv code when country is BE' do
      result = tested_class.get_crv_organisation_code "BE"
      expect(result).to eq("crv.be")
    end
  end

  context 'perform_service_message_validation' do
    it 'should throw an crv error on an error' do
     serviceMessage = ::CrvApiClient::Api::Animals::ServiceMessage.new("200","200","some error","somme  type")
     expect{tested_class.send(:perform_service_message_validation,serviceMessage)}.to raise_error(::CrvApiClient::Errors::CrvRequestError)
    end

    it 'should throw an crv error on an error on an Reproduction Service message' do
      serviceMessage = ::CrvApiClient::Api::Reproduction::ServiceMessage.new("200","200","some error","somme  type")
      expect{tested_class.send(:perform_service_message_validation,serviceMessage)}.to raise_error(::CrvApiClient::Errors::CrvRequestError)
    end
  end


  context 'context_message' do
    it 'should return a context message' do
      username = "abc"
      password = "efg"
      keeper = ::CrvApiClient::Api::Reproduction::ParticipantId.new("a","b","c")
      animalid = ::CrvApiClient::Api::Reproduction::AnimalId.new("b","c")
      contextmessageDetail = [::CrvApiClient::Api::Reproduction::ContextMessageDetail.new("",""), ::CrvApiClient::Api::Reproduction::ContextMessageDetail.new("b","c")]
      processCode = "102"
      organisation = ::CrvApiClient::Api::Reproduction::ParticipantId.new("or","b","c")
      crv_customer = ::CrvApiClient::Api::Reproduction::ParticipantId.new("10106605","PAR","NLD")
      crv_provider = ::CrvApiClient::Api::Reproduction::ParticipantId.new("2323","test","asd")

      expect(tested_class).to receive(:endpoint_context_message).and_return(::CrvApiClient::Api::Reproduction::ContextMessage)
      message = tested_class.send(:context_message, keeper,
                                       animal: animalid,
                                       username: username,
                                       password: password,
                                       context_message_detail: contextmessageDetail,
                                       process_code: processCode,
                                       customer: crv_customer,
                                       provider: crv_provider,
                                       organisation: organisation)

      expect(message.animal).to be(animalid)
      expect(message.keeper).to be(keeper)
      expect(message.contextMessageDetail).to be(contextmessageDetail)
      expect(message.processCode).to be (processCode)
      expect(message.username).to be(username)
      expect(message.password).to be(password)
      expect(message.organisation).to be(organisation)
      expect(message.customer).to be(crv_customer)
      expect(message.provider).to be(crv_provider)
    end
  end

  context 'keeper_and_context_message_detail' do
    it 'should return an keeper and context message' do
      crv_id = 29
      country_code = "NL"
      keeper = ::CrvApiClient::Api::Reproduction::ParticipantId.new(crv_id,"",country_code)
      expect(tested_class).to receive(:endpoint_context_message_detail).and_return(::CrvApiClient::Api::Reproduction::ContextMessageDetail)
      expect(tested_class).to receive(:get_keeper).with(crv_id.to_s,country_code).and_return(keeper)

      keeper, context_message = tested_class.send(:keeper_and_context_message_detail, crv_id, country_code)
      expect(keeper).to be_a(::CrvApiClient::Api::Reproduction::ParticipantId)
      expect(keeper).to eq(keeper)
      expect(context_message).to be_a(::CrvApiClient::Api::Reproduction::ContextMessageDetail)
    end
  end

end

